from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage


class GarpixPaginator(Paginator):

    def __init__(self, object_list, per_page, orphans=0,
                 allow_empty_first_page=True, neighbors=4):
        super().__init__(object_list, per_page, orphans, allow_empty_first_page)
        self.current_page = 1
        self.neighbors = int(neighbors)

    def get_page(self, number):
        try:
            number = self.validate_number(number)
        except PageNotAnInteger:
            number = 1
        except EmptyPage:
            number = self.num_pages
        self.current_page = number
        return self.page(number)

    @property
    def page_range_beauty(self):
        left_page = self.current_page - self.neighbors
        if left_page < 1:
            left_page = 1
        right_page = self.current_page + self.neighbors
        if right_page > self.num_pages:
            right_page = self.num_pages
        page_range_list = list(map(lambda x: (x, x), range(left_page, right_page + 1)))
        if left_page > 2:
            page_range_list.insert(0, (left_page - 1, '...'))
        if left_page != 1:
            page_range_list.insert(0, (1, 1))
        if right_page < self.num_pages - 1:
            page_range_list.append((right_page + 1, '...'))
        if right_page != self.num_pages:
            page_range_list.append((self.num_pages, self.num_pages))
        return page_range_list



from django import template

register = template.Library()


@register.simple_tag
def url_replace(request, field, value):
    """
    Подмена одного значения в dict на другое в Django-шаблонах.
    Применять можно так:
        {% url_replace request 'page' page_num %}
    """
    dict_ = request.GET.copy()
    dict_[field] = value
    return dict_.urlencode()



from .active_manager import ActiveManager
from .available_manager import AvailableManager
from .polymorphic_active_manager import PolymorphicActiveManager
from .polymorphic_available_manager import PolymorphicAvailableManager



from polymorphic.managers import PolymorphicManager


class PolymorphicAvailableManager(PolymorphicManager):
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True, is_deleted=False)



from .delete_mixin import DeleteMixin
from .active_mixin import ActiveMixin
from polymorphic_tree.models import PolymorphicMPTTModel
from ..managers.polymorphic_available_manager import PolymorphicAvailableManager


class PolymorphicAvailableMixin(ActiveMixin, DeleteMixin, PolymorphicMPTTModel):
    available_objects = PolymorphicAvailableManager()

    class Meta:
        abstract = True
